Attribute VB_Name = "modSheetFunctions"
Option Explicit
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' modSheetFunctions
' By Chip Pearson, chip@cpearson.com, www.cpearson.com/Excel/SheetNames.aspx
' Created 12-February-2011
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Function SheetNames(Optional R As Range, _
        Optional VisibleOnly As Boolean = False) As String()
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetNames
' This returns an array of sheet names in the workbook containing
' the range reference R. If R is omitted, the calling workbook
' is used. If VisibleOnly is True, only names of visible worksheets
' are returned. If VisibleOnly is False, all worksheet names are returned.
' The result is an array of sheet names. The size of the array
' is the greater of Application.Caller.Cells.Count and
' Worksheets.Count.
' To display the list in one row spanning several columns, array enter
' =SheetNames() into the range of cells. To display the list in
' one column spanning several rows, array enter =TRANSPOSE(SheetNames())
' into the cells.
' If the range into which the formula is entered contains more than
' one row AND more than one column, #REF is returned.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim SS() As String
    Dim WB As Workbook
    Dim WS As Worksheet
    Dim N As Long
   
    If Application.Caller.Rows.Count > 1 And _
        Application.Caller.Columns.Count > 1 Then
            SheetNames = CVErr(xlErrRef)
            Exit Function
    End If
    If R Is Nothing Then
        Set WB = Application.Caller.Worksheet.Parent
    Else
        Set WB = R.Worksheet.Parent
    End If
    N = Application.WorksheetFunction.Max( _
        WB.Worksheets.Count, Application.Caller.Cells.Count)
    ReDim SS(1 To N)
    N = 0
    For Each WS In WB.Worksheets
        If VisibleOnly = False Or _
            (VisibleOnly = True And WS.Visible = xlSheetVisible) Then
            N = N + 1
            SS(N) = WS.Name
        End If
    Next WS
    SheetNames = SS
End Function

Public Function SheetName(Optional R As Range) As String
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetName
' This returns a worksheet name. If R is present, it returns
' the name of the worksheet containing R. If R is omitted,
' it returns Application.Caller.Worksheet.Name.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If R Is Nothing Then
        SheetName = Application.Caller.Worksheet.Name
    Else
        SheetName = R.Worksheet.Name
    End If
End Function

Public Function SheetNameOfIndex(Index As Long, Optional R As Range) As Variant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetNameOfIndex
' This returns the worksheet whose index is Index. Returns #REF!
' if index is out of bounds.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim WB As Workbook
    If R Is Nothing Then
        Set WB = Application.Caller.Worksheet.Parent
    Else
        Set WB = R.Worksheet.Parent
    End If
    If Index <= 0 Or Index > WB.Worksheets.Count Then
        SheetNameOfIndex = CVErr(xlErrRef)
        Exit Function
    End If
    Err.Clear
    On Error Resume Next
    SheetNameOfIndex = WB.Worksheets(Index).Name
    If Err.Number <> 0 Then
        SheetNameOfIndex = CVErr(xlErrRef)
    End If
End Function

Public Function SheetExists(SheetName As String, Optional R As Range) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetExists
' This tests whether SheetName exists in a workbook. If R is
' present, the workbook containing R is used. If R is omitted,
' Application.Caller.Worksheet.Parent is used.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim WS As Worksheet
    Dim WB As Workbook
    If R Is Nothing Then
        Set WB = Application.Caller.Worksheet.Parent
    Else
        Set WB = R.Worksheet.Parent
    End If
    On Error Resume Next
    Err.Clear
    Set WS = WB.Worksheets(SheetName)
    If Err.Number = 0 Then
        SheetExists = True
    Else
        SheetExists = False
    End If
End Function

Public Function SheetIndex(Optional R As Range) As Long
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetIndex
' This returns the Index of the worksheet containing R. If R
' is omitted, the Index of the worksheet from which the function
' is called is returned.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If R Is Nothing Then
        SheetIndex = Application.Caller.Worksheet.Index
    Else
        SheetIndex = R.Worksheet.Index
    End If
End Function

Public Function SheetCount(Optional R As Range, Optional VisibleOnly As Boolean) As Long
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetCount
' Returns the count of worksheets. If R is present, the count
' of worksheets in the workbook containing R is returned. If R
' is omitted, the count of worksheets in the workbook calling
' the function is returned. If VisibleOnly is True, only visible
' worksheets are counted. If VisibleOnly is False, the count
' of all worksheets is returned.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim WB As Workbook
    Dim WS As Worksheet
    Dim N As Long
    If R Is Nothing Then
        Set WB = Application.Caller.Worksheet.Parent
    Else
        Set WB = R.Worksheet.Parent
    End If
    For Each WS In WB.Worksheets
        If VisibleOnly = False Or _
            (VisibleOnly = True And WS.Visible = xlSheetVisible) Then
           N = N + 1
        End If
    Next WS
    SheetCount = N
End Function


Public Function SheetNameOffset(N As Long, Optional R As Range, _
    Optional Wrap As Boolean) As Variant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetNameOffset
' This returns the sheet name that is N sheets before or after
' a specified worksheet WS. If R is present, WS is the worksheet
' containing R. If R is omitted, WS is Application.Caller.Worksheet.
' The offset N is 0 based. SheetNameOffset(0) is WS itself. If
' N < 0, the returned sheet name is Abs(N) worksheets before WS. If
' N > 0, the returned sheet name is Abs(N) worksheets after WS. In
' either case, if N is greater than the offset to the first or
' last sheet, the function returns #REF. If Wrap is True and the
' first sheet is greater than N, the search wraps to the last sheet.
' If Wrap is True and the last sheet is less than N, the search
' wraps to the first sheet.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim M As Long
    Dim WS As Worksheet
    Dim WB As Workbook
    
    If R Is Nothing Then
        Set WS = Application.Caller.Worksheet
    Else
        Set WS = R.Worksheet
    End If
    Set WB = WS.Parent
    If N = 0 Then
        SheetNameOffset = WS.Name
        Exit Function
    End If
    On Error Resume Next
    If N < 0 Then
        For M = -1 To N Step -1
            Set WS = WS.Previous
            If WS Is Nothing Then
                If Wrap = True Then
                    With WB.Worksheets
                        Set WS = .Item(.Count)
                    End With
                Else
                    SheetNameOffset = CVErr(xlErrRef)
                    Exit Function
                End If
            End If
        Next M
    Else
        For M = 1 To N
            Set WS = WS.Next
            If WS Is Nothing Then
                If Wrap = True Then
                    With WB.Worksheets
                        Set WS = .Item(1)
                    End With
                Else
                    SheetNameOffset = CVErr(xlErrRef)
                    Exit Function
                End If
            End If
        Next M
    End If
    SheetNameOffset = WS.Name
End Function


Public Function WorkbookName(Optional R As Range, Optional FullName As Boolean) As String
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' WorkbookName
' This returns the name or full name of a workbook. If R is
' present, it returns the name of the workbook containing R.
' If R is omitted, it returns the name of the workbook from
' which it was called. If FullName is omitted or False, the
' Name is returned. If FullName is True, the FullName is returned.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If R Is Nothing Then
        If FullName = False Then
            WorkbookName = Application.Caller.Worksheet.Parent.Name
        Else
            WorkbookName = Application.Caller.Worksheet.Parent.FullName
        End If
    Else
        If FullName = False Then
            WorkbookName = R.Worksheet.Parent.Name
        Else
            WorkbookName = R.Worksheet.Parent.FullName
        End If
    End If
End Function

Public Function WorkbookNames(Optional FullName As Boolean, _
        Optional SavedOnly As Boolean, _
        Optional VisibleOnly As Boolean) As String()
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' WorkbookNames
' This returns an array of strings containing workbook names.
' If FullName is True, the workbook's FullName is returned. If
' FullName is False, the workbook's Name is returned. If SavedOnly
' is True, only workbooks that have been saved to disc are returned.
' If SavedOnly is False, all workbook names are returned. If
' VisibleOnly is True, only the names of visible workbooks are
' returned. If VisibleOnly is False, all workbook names are
' returned.
' To display the list in one row spanning several columns, array enter
' =WorkbookNames() into the range of cells. To display the list in
' one column spanning several rows, array enter =TRANSPOSE(WorkbookNames())
' into the cells.
' If the range into which the formula is entered contains more than
' one row AND more than one column, #REF is returned.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim Arr() As String
    Dim N As Long
    Dim WB As Workbook
    
    If Application.Caller.Rows.Count > 1 And _
            Application.Caller.Columns.Count > 1 Then
        WorkbookNames = CVErr(xlErrRef)
        Exit Function
    End If
    
    N = Application.WorksheetFunction.Max( _
        Application.Workbooks.Count, Application.Caller.Cells.Count)
    ReDim Arr(1 To N)
    N = 0
    For Each WB In Application.Workbooks
        If SavedOnly = False Or (SavedOnly = True And Len(WB.Path) > 0) Then
            If VisibleOnly = False Or _
                (VisibleOnly = True And WB.Windows(1).Visible = True) Then
                N = N + 1
                If FullName = True Then
                    Arr(N) = WB.FullName
                Else
                    Arr(N) = WB.Name
                End If
            End If
        End If
    Next WB
    WorkbookNames = Arr
End Function

Public Function WorkbookCount(Optional VisibleOnly As Boolean) As Long
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' WorkbookCount
' This returns the count of all open workbooks. If VisibleOnly
' is True, only visible workbooks are counted. If VisibleOnly is
' False, all workbooks are counted.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim WB As Workbook
    Dim N As Long
    For Each WB In Application.Workbooks
        If VisibleOnly = False Or (VisibleOnly = True And WB.Windows(1).Visible = True) Then
            N = N + 1
        End If
    Next WB
    WorkbookCount = N
End Function

Public Function WorkbookPath(Optional R As Range) As String
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' WorkbookPath
' This returns the path of the workbook containing range R.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
If R Is Nothing Then
    WorkbookPath = Application.Caller.Worksheet.Parent.Path
Else
    WorkbookPath = R.Worksheet.Parent.Path
End If
End Function

Public Function SheetVisible(SheetName As String, Optional R As Range) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SheetVisible
' Returns True or False indicating whether worksheet SheetName
' is visible. If R is included, that range's workbook is seached
' for SheetName. Otherwise, the calling workbook is searched.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim WB As Workbook
Dim WS As Worksheet

If R Is Nothing Then
    Set WB = Application.Caller.Worksheet.Parent
Else
    Set WB = R.Worksheet.Parent
End If
On Error Resume Next
Err.Clear
Set WS = WB.Worksheets(SheetName)
If Err.Number <> 0 Then
    SheetVisible = False
Else
    If WS.Visible = xlSheetVisible Then
        SheetVisible = True
    Else
        SheetVisible = False
    End If
End If

End Function

Public Function NextSheetName(Optional R As Range) As Variant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' NextSheetName
' Returns the name of the next sheet. If R is provided, it
' returns the sheet following the sheet containing R. If R
' is omitted, it returns the sheet after the sheet from which
' it was called. If there is no next worksheet, the result is
' #REF.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim S As String
Dim WS As Worksheet
If R Is Nothing Then
    Set WS = Application.Caller.Worksheet
Else
    Set WS = R.Worksheet
End If
On Error Resume Next
S = WS.Next.Name
If Err.Number = 0 Then
    NextSheetName = S
Else
    NextSheetName = CVErr(xlErrRef)
End If

End Function

Public Function PreviousSheetName(Optional R As Range) As Variant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' PreviousSheetName
' Returns the name of the previous sheet. If R is provided, it
' returns the sheet before the sheet containing R. If R
' is omitted, it returns the sheet before the sheet from which
' it was called. If there is no previous sheet, the result is
' #REF.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim S As String
Dim WS As Worksheet
If R Is Nothing Then
    Set WS = Application.Caller.Worksheet
Else
    Set WS = R.Worksheet
End If
On Error Resume Next
Err.Clear
S = WS.Previous.Name
If Err.Number = 0 Then
    PreviousSheetName = S
Else
    PreviousSheetName = CVErr(xlErrRef)
End If

End Function

